package com.perforce.hws.swagger.asciidoc.models;

import io.swagger.models.RefModel;
import io.swagger.models.parameters.BodyParameter;
import io.swagger.models.parameters.Parameter;
import io.swagger.models.parameters.PathParameter;
import io.swagger.models.parameters.QueryParameter;

/**
 * Routes can contain parameters in paths, the body, or query string.
 *
 * In languages, these parameters typically appear as method parameters, with subtly different
 * types, which is where the crazy is.
 */
public class ParamHelper implements LinkHelpers {

    // The underlying Swagger definition
    private Parameter parameter;

    public ParamHelper(Parameter parameter) {
        this.parameter = parameter;
    }

    public String getName() {
        return this.parameter.getName();
    }

    public boolean isPathParam() {
        return this.parameter.getIn().equals("path");
    }

    public boolean isQueryParam() {
        return this.parameter.getIn().equals("query");
    }

    public boolean isBodyParam() {
        return this.parameter.getIn().equals("body");
    }

    public String getJavaTypeRef() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    return javaModelRef(refModel.getSimpleRef());
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return toSimpleJavaType(pathParameter.getType(), pathParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return toSimpleJavaType(queryParameter.getType(), queryParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());
    }

    public String getJavaType() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    return refModel.getSimpleRef();
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return toSimpleJavaType(pathParameter.getType(), pathParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return toSimpleJavaType(queryParameter.getType(), queryParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());
    }

    public String getJavascriptTypeRef() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    return javascriptModelRef(refModel.getSimpleRef());
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return toSimpleJavascriptType(pathParameter.getType(), pathParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return toSimpleJavascriptType(queryParameter.getType(), queryParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());
    }

    public String getRubyType() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    return refModel.getSimpleRef();
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return toSimpleRubyType(pathParameter.getType(), pathParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return toSimpleRubyType(queryParameter.getType(), queryParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());
    }

    public String getPhpTypeRef() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    return phpModelRef(refModel.getSimpleRef());
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return toSimplePhpType(pathParameter.getType(), pathParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return toSimplePhpType(queryParameter.getType(), queryParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());
    }


    public String getPythonType() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    return refModel.getSimpleRef();
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return toSimplePythonType(pathParameter.getType(), pathParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return toSimplePythonType(queryParameter.getType(), queryParameter.getFormat());
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());
    }

    public String getType() {
        if (this.parameter instanceof BodyParameter) {
            BodyParameter bodyParameter = (BodyParameter)parameter;
            if (bodyParameter.getSchema() instanceof RefModel) {
                RefModel refModel = (RefModel)bodyParameter.getSchema();
                if (refModel.getSimpleRef() != null) {
                    // This may need to be "javascript" schema specific
                    return refModel.getSimpleRef();
                } else {
                    throw new IllegalStateException("Can't handle: " + refModel);
                }
            } else {
                throw new IllegalStateException("Don't know schema type: " + bodyParameter.getSchema());
            }
        } else if (this.parameter instanceof PathParameter) {
            PathParameter pathParameter = (PathParameter)parameter;
            if (pathParameter.getType() != null) {
                return pathParameter.getType();
            } else {
                throw new IllegalStateException("Don't know how to handle path parameter: " + pathParameter);
            }
        } else if (this.parameter instanceof QueryParameter) {
            QueryParameter queryParameter = (QueryParameter) parameter;
            if (queryParameter.getType() != null) {
                return queryParameter.getType();
            } else {
                throw new IllegalStateException("Don't know how to represent query parameter type: " + queryParameter);
            }
        }
        throw new IllegalStateException("Don't know parameter type: " + this.parameter.getClass().getName());

    }

    public String getDescription() {
        return this.parameter.getDescription();
    }

    public boolean isRequired() {
        return this.parameter.getRequired();
    }

    public static String toSimpleJavaType(String type, String format) {
        if (format != null) {
            if (format.equals("date-time")) {
                return "Date";
            } else {
                throw new IllegalStateException("Don't know format " + format);
            }
        } else {
            if (type.equals("string")) {
                return "String";
            } else if (type.equals("integer")) {
                return "Integer";
            } else {
                throw new IllegalStateException("Don't know simple type " + type);
            }
        }
    }


    public static String toSimpleJavascriptType(String type, String format) {
        if (format != null) {
            if (format.equals("date-time")) {
                return "Date";
            } else {
                throw new IllegalStateException("Don't know format " + format);
            }
        } else {
            if (type.equals("string")) {
                return "String";
            } else if (type.equals("integer")) {
                return "Number";
            } else {
                throw new IllegalStateException("Don't know simple type " + type);
            }
        }
    }



    public static String toSimpleRubyType(String type, String format) {
        if (format != null) {
            if (format.equals("date-time")) {
                return "Date";
            } else {
                throw new IllegalStateException("Don't know format " + format);
            }
        } else {
            if (type.equals("string")) {
                return "String";
            } else if (type.equals("integer")) {
                return "Integer";
            } else {
                throw new IllegalStateException("Don't know simple type " + type);
            }
        }
    }

    public static String toSimplePhpType(String type, String format) {
        if (format != null) {
            if (format.equals("date-time")) {
                return "\\DateTime";
            } else {
                throw new IllegalStateException("Don't know format " + format);
            }
        } else {
            if (type.equals("string")) {
                return "string";
            } else if (type.equals("integer")) {
                return "int";
            } else if (type.equals("boolean")) {
                return "bool";
            } else {
                throw new IllegalStateException("Don't know simple type " + type);
            }
        }
    }

    public static String toSimplePythonType(String type, String format) {
        if (format != null) {
            if (format.equals("date-time")) {
                return "datetime";
            } else {
                throw new IllegalStateException("Don't know format " + format);
            }
        } else {
            if (type.equals("string")) {
                return "str";
            } else if (type.equals("integer")) {
                return "int";
            } else if (type.equals("boolean")) {
                return "bool";
            } else {
                throw new IllegalStateException("Don't know simple type " + type);
            }
        }
    }
}
