package com.perforce.hws.swagger.spark;

import io.swagger.codegen.*;
import io.swagger.models.properties.*;

import java.util.*;
import java.io.File;

public class SparkGenerator extends DefaultCodegen implements CodegenConfig {

	// source folder where to write the files
	protected String sourceFolder = "";
	protected String apiVersion = "1.0.0";

	protected String basePackage = "com.perforce.hws.spark";
	protected String baseName = "SparkServerPlugin";

	/**
	 * Configures the type of generator.
	 *
	 * @return the CodegenType for this generator
	 * @see io.swagger.codegen.CodegenType
	 */
	public CodegenType getTag() {
		return CodegenType.CLIENT;
	}

	/**
	 * Configures a friendly name for the generator.  This will be used by the generator
	 * to select the library with the -l flag.
	 *
	 * @return the friendly name for the generator
	 */
	public String getName() {
		return "spark";
	}

	/**
	 * Returns human-friendly help for the generator.  Provide the consumer with help
	 * tips, parameters here
	 *
	 * @return A string value for the help message
	 */
	public String getHelp() {
		return "Generates Spark server stubs.";
	}

	public SparkGenerator() {
		super();

		outputFolder = "generated-code/spark";

		// no model
		modelTemplateFiles.clear();
		apiTemplateFiles.put("api.mustache", ".java");

	//	supportingFiles.add(new SupportingFile("build.mustache", "", "build.gradle"));
	//	supportingFiles.add(new SupportingFile("plugin.mustache", (sourceFolder + File.separator + basePackage).replace(".", java.io.File.separator), "SparkServerPlugin.java"));

		embeddedTemplateDir = templateDir = "spark";

		apiPackage = "com.perforce.hws.spark.api";
		modelPackage = "com.perforce.hws.spark.model";

		setReservedWordsLowerCase(
				Arrays.asList(
						// used as internal variables, can collide with parameter names
						"localVarPath", "localVarQueryParams", "localVarHeaderParams", "localVarFormParams",
						"localVarPostBody", "localVarAccepts", "localVarAccept", "localVarContentTypes",
						"localVarContentType", "localVarAuthNames", "localReturnType",
						"ApiClient", "ApiException", "ApiResponse", "Configuration", "StringUtil",

						// language reserved words
						"abstract", "continue", "for", "new", "switch", "assert",
						"default", "if", "package", "synchronized", "boolean", "do", "goto", "private",
						"this", "break", "double", "implements", "protected", "throw", "byte", "else",
						"import", "public", "throws", "case", "enum", "instanceof", "return", "transient",
						"catch", "extends", "int", "short", "try", "char", "final", "interface", "static",
						"void", "class", "finally", "long", "strictfp", "volatile", "const", "float",
						"native", "super", "while")
		);

		languageSpecificPrimitives = new HashSet<String>(
				Arrays.asList(
						"String",
						"boolean",
						"Boolean",
						"Double",
						"Integer",
						"Long",
						"Float",
						"Object",
						"byte[]")
		);

		instantiationTypes.put("array", "ArrayList");
		instantiationTypes.put("map", "HashMap");
		typeMapping.put("date", "Date");
		typeMapping.put("file", "File");
		typeMapping.put("UUID", "String");

		additionalProperties.put("apiVersion", apiVersion);
		additionalProperties.put("basePackage", basePackage);
		additionalProperties.put("baseName", baseName);

		/**
		 * Supporting Files.  You can write single files for the generator with the
		 * entire object tree available.  If the input file has a suffix of `.mustache
		 * it will be processed by the template engine.  Otherwise, it will be copied

		 supportingFiles.add(new SupportingFile("myFile.mustache",   // the input template or file
		 "",                                                       // the destination folder, relative `outputFolder`
		 "myFile.sample")                                          // the output file
		 );*/
	}

	@Override
	public String escapeReservedWord(String name) {
		return "_" + name;
	}

	/**
	 * Location to write model files.  You can use the modelPackage() as defined when the class is
	 * instantiated
	 */
	public String modelFileFolder() {
		return outputFolder + "/" + sourceFolder + "/" + modelPackage().replace('.', File.separatorChar);
	}

	/**
	 * Location to write api files.  You can use the apiPackage() as defined when the class is
	 * instantiated
	 */
	@Override
	public String apiFileFolder() {
		return outputFolder + "/" + sourceFolder + "/" + apiPackage().replace('.', File.separatorChar);
	}

	/**
	 * Optional - type declaration.  This is a String which is used by the templates to instantiate your
	 * types.  There is typically special handling for different property types
	 *
	 * @return a string value used as the `dataType` field for model templates, `returnType` for api templates
	 */
	@Override
	public String getTypeDeclaration(Property p) {
		if (p instanceof ArrayProperty) {
			ArrayProperty ap = (ArrayProperty) p;
			Property inner = ap.getItems();
			return getSwaggerType(p) + "[" + getTypeDeclaration(inner) + "]";
		} else if (p instanceof MapProperty) {
			MapProperty mp = (MapProperty) p;
			Property inner = mp.getAdditionalProperties();
			return getSwaggerType(p) + "[String, " + getTypeDeclaration(inner) + "]";
		}
		return super.getTypeDeclaration(p);
	}

	/**
	 * Optional - swagger type conversion.  This is used to map swagger types in a `Property` into
	 * either language specific types via `typeMapping` or into complex models if there is not a mapping.
	 *
	 * @return a string value of the type or complex model for this property
	 * @see io.swagger.models.properties.Property
	 */
	@Override
	public String getSwaggerType(Property p) {
		String swaggerType = super.getSwaggerType(p);
		String type = null;
		if (typeMapping.containsKey(swaggerType)) {
			type = typeMapping.get(swaggerType);
			if (languageSpecificPrimitives.contains(type))
				return toModelName(type);
		} else
			type = swaggerType;
		return toModelName(type);
	}

	@Override
	public Map<String, Object> postProcessOperations(Map<String, Object> operations) {
		Map<String, Object> objs = (Map<String, Object>) operations.get("operations");
		List<CodegenOperation> ops = (List<CodegenOperation>) objs.get("operation");
		for (CodegenOperation op : ops) {
			// Convert httpMethod to lower case, e.g. "get", "post"
			op.httpMethod = op.httpMethod.toLowerCase();

			// Change path parameters {var} to :var
			op.path = op.path.replaceAll("\\{", ":");
			op.path = op.path.replaceAll("\\}", "");

			op.operationId = camelize(op.operationId);
		}
		return operations;
	}

	@Override
	public String toApiName(String name) {
		return "P4Api";
	}
}